# -*- coding: utf-8 -*-
## A Plone product to generate quiz. A quiz is a list of questions with the appropriate answer(s).
## Content managers can build quiz easily and present users a report about the errors they did.
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
<docstring de votre module>
"""
__docformat__ = 'restructuredtext'


from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.Archetypes.atapi import *
from Products.Archetypes.Field import encode, decode
from Products.DataGridField import DataGridField, DataGridWidget


from Products.PloneQuiz.config import *
##code-section module-header #fill in your manual code here
##/code-section module-header

schema=Schema((
    StringField('question',
        required=True,
        searchable = False,
        widget=StringWidget(
            label='Question',
            label_msgid='quiz_label_question',
            description_msgid='quiz_help_question',
            i18n_domain='quiz',
        )
    ),

    DataGridField('answers',
        required = True,
        searchable = False,
        widget=DataGridWidget(
            label='Answers',
            macro='answers_widget',
            label_msgid='quiz_label_answers',
            description_msgid='quiz_help_answers',
            i18n_domain='quiz',
        ),
                  columns=('answer',CORRECT_ANSWER_STRING),   #using global constant, datagridfield can only store strings
    ),

    TextField('rightAnswerMessage',
        searchable = False,
        widget=TextAreaWidget(
            label='Rightanswermessage',
            label_msgid='quiz_label_rightAnswerMessage',
            description_msgid='quiz_help_rightAnswerMessage',
            i18n_domain='quiz',

        )
    ),

    TextField('wrongAnswerMessage',
        searchable = False,
        widget=TextAreaWidget(
            label='Wronganswermessage',
            label_msgid='quiz_label_wrongAnswerMessage',
            description_msgid='quiz_help_wrongAnswerMessage',
            i18n_domain='quiz',
        )
    ),

    StringField('relatedPage',
        searchable = False,
        widget=StringWidget(
            label='Relatedpage',
            label_msgid='quiz_label_relatedPage',
            description_msgid='quiz_help_relatedPage',
            i18n_domain='quiz',
            macro='relatedpage_template',
        ),
        validators=('isURL',)
    ),

    ImageField('image',
        sizes={ 'mini' : (80,80),
                'thumb' : (150,150),
                'normal' : (200,200),
                'big' : (300,300)
                },
        widget=ImageWidget(
            label='Image',
            label_msgid='quiz_label_image',
            description_msgid='quiz_help_image',
            i18n_domain='quiz',
        ),
        storage=AttributeStorage()
    ),

),
)


##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

QuizQA_schema = BaseSchema + \
    schema

QuizQA_schema['title'].required = 0
QuizQA_schema['title'].widget.visible['view'] = 'visible' # !!
QuizQA_schema['title'].widget.visible['edit'] = 'invisible' # !! 
QuizQA_schema['id'].widget.visible['view'] = 'invisible' # !!
QuizQA_schema['id'].widget.visible['edit'] = 'invisible' # !!


##code-section after-schema #fill in your manual code here
##/code-section after-schema

class QuizQA(BaseContent):
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseContent,'__implements__',()),)


    # This name appears in the 'add' box
    archetype_name             = 'Question'

    meta_type                  = 'QuizQA'
    portal_type                = 'QuizQA'
    allowed_content_types      = []
    filter_content_types       = 0
    global_allow               = 0
    allow_discussion           = 0
    content_icon               = 'question.png'
    immediate_view             = 'base_view'
    default_view               = 'base_view'
    suppl_views                = ()
    typeDescription            = "QuizQA"
    typeDescMsgId              = 'description_edit_quizqa'

    schema = QuizQA_schema

    #Methods
    security.declareProtected(CMFCorePermissions.View, 'Title')
    def Title(self,):
        """Retutn content of 'question' field as title
        """
        question = decode(self.getQuestion(), self)
        return encode(question, self)

    security.declareProtected(CMFCorePermissions.View, 'getUid')
    def getUid(self):
        return self.UID()

    security.declareProtected(CMFCorePermissions.View, 'getRightAnswerMessage')
    def getRightAnswerMessage(self):
        """
        returns default right answer
        """
        rightAnswerMessage = self.getField('rightAnswerMessage').get(self)
        if not rightAnswerMessage:
            return self.aq_parent.getDefaultRightAnswerMessage()
        else:
            return rightAnswerMessage

    security.declareProtected(CMFCorePermissions.View, 'getWrongAnswerMessage')
    def getWrongAnswerMessage(self):
        """the same that above"""

        wrongAnswerMessage = self.getField('wrongAnswerMessage').get(self)
        if not wrongAnswerMessage:
            return self.aq_parent.getDefaultWrongAnswerMessage()
        else:
            return wrongAnswerMessage

    security.declareProtected(CMFCorePermissions.View, 'getRelatedPage')
    def getRelatedPage(self):
        """ returns the related page url if there is one
             or returns the defaul one if set
        """

        relatedPage = self.getField('relatedPage').get(self)
        if not relatedPage:
            return self.aq_parent.getDefaultRelatedPage()
        else:
            return relatedPage

    security.declareProtected(CMFCorePermissions.View, 'getCorrectAnswers')
    def getCorrectAnswers(self):
        """get correct answers"""
        return ', '.join([x['answer'] for x in self.getAnswers() if x[CORRECT_ANSWER_STRING]])

    security.declareProtected(CMFCorePermissions.View, 'setChecked')
    def setChecked(self,index,answer):
        """Checks if this answer was already checked
        """
        id = self.aq_parent.getId()
        if self.aq_parent.getShowOnOnePage():
            if self.REQUEST.form.get('answer',None) and self.REQUEST.form['answer'].has_key(str(index)) and answer in self.REQUEST.form['answer'][str(index)]:
                return "checked"
        else:
            return ""

    security.declareProtected(CMFCorePermissions.View, 'validateMyAnswer')
    def validateMyAnswer(self,providedAnswers):
        """compares provided answers for with correct answers for one question and returns true or false.
        """
        if providedAnswers[0] == "not answered":
            providedAnswers[0] = self.translate('quiz_void_answer', default='no answer provided', domain='quiz')
        correctAnswers=[x['answer'] for x in self.getAnswers() if x[CORRECT_ANSWER_STRING]]
        correctAnswers.sort()
        providedAnswers.sort()
        if correctAnswers == providedAnswers:
            return True
        else:
            return False

    security.declareProtected(CMFCorePermissions.View, 'hasJustOneCorrectAnswer')
    def hasJustOneCorrectAnswer(self):
        answers = [x['answer'] for x in self.getAnswers() if x[CORRECT_ANSWER_STRING]]
        if len(answers) == 1:
            return True
        else:
            return False

    security.declareProtected(CMFCorePermissions.View, 'tag')
    def tag(self, **kwargs):
        """Generate image tag using the api of the ImageField
        """
        if 'title' not in kwargs:
            kwargs['title'] = self.Title()
        return self.getField('image').tag(self, **kwargs)

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header
    security.declareProtected(CMFCorePermissions.View, 'post_validate')
    def post_validate(self, REQUEST=None, errors=None):
        """Extra validation
        """
        # Check answers
        answers = REQUEST.form.get('answers', None)
        error = False
        if answers is None:
            error = True
        elif len(answers) < 3:
            error = True
        else:
            has_good_response = False
            for answer in answers[:-1]:
                if answer['answer'] == '':
                    error = True
                    break
                if answer.has_key('correct_edit_string'):
                    has_good_response = True
            if not has_good_response:
                error = True
        if error:
            response = REQUEST.RESPONSE
            response.setHeader('Content-Type', 'text/html; charset=%s' % self.getCharset() )
            errors['answers'] = self.translate('quiz_answers_error',
                                       default='Please provide at least two answers. At least one of them should be correct',
                                       domain='quiz')
        return

# # quiz_answers_error






registerType(QuizQA,PROJECTNAME)
# end of class QuizQA

##code-section module-footer #fill in your manual code here
##/code-section module-footer



